@@ -24,7 +24,7 @@ Insert and find:
 >>> find(lambda v: v["age"] >= 20)[0]["name"]
 'Sabu'
 """
-# pylint: disable=C0103,W0622,W0603
+# pylint: disable=C0103,C301,W0622,W0603
 import sqlite3
 import time
 import json
@@ -32,9 +32,10 @@ import json
 db = None
 cache_db = {}
 CACHE_KEYS = {}
-sqls = {}
+SQLS = {}
 MEMORY_FILE = ':memory:'
 cur_filename = MEMORY_FILE
+SQLITE_MAX_INT=9223372036854775807
 # SQL template
 SQLS_TEMPLATE = {
     # kvs
@@ -64,41 +65,41 @@ SQLS_TEMPLATE = {
     )
     ''',
     'select_doc': 'SELECT value, id FROM doc__TABLE_NAME__',
-    'recent_doc': 'SELECT value, id FROM doc__TABLE_NAME__ ORDER BY id DESC LIMIT ?',
+    'select_doc_desc': 'SELECT value, id FROM doc__TABLE_NAME__ WHERE id <= ? ORDER BY id DESC LIMIT ?',
+    'select_doc_asc': 'SELECT value, id FROM doc__TABLE_NAME__ WHERE id >= ? ORDER BY id ASC LIMIT ?',
+    'recent_doc': 'SELECT value, id FROM doc__TABLE_NAME__ ORDER BY id DESC LIMIT ? OFFSET ?',
     'get_doc_by_id': 'SELECT value, id FROM doc__TABLE_NAME__ WHERE id=?',
     'insert_doc': 'INSERT INTO doc__TABLE_NAME__ (value, ctime, mtime) VALUES (?, ?, ?)',
     'update_doc': 'UPDATE doc__TABLE_NAME__ SET value=?, mtime=? WHERE id=?',
     'delete_doc': 'DELETE FROM doc__TABLE_NAME__ WHERE id=?',
     'clear_doc': 'DELETE FROM doc__TABLE_NAME__',
+    'count_doc': 'SELECT count(id) FROM doc__TABLE_NAME__',
 }
 
 def connect(filename = ':memory:', table_name='kudb'):
     """Connect to database"""
-    global sqls, cur_filename
+    global SQLS, cur_filename, db
     # generate sqls
-    sqls = {}
+    SQLS = {}
     for key, val in SQLS_TEMPLATE.items():
-        sqls[key] = val.replace('__TABLE_NAME__', table_name)
-    # connect to sqlite3
-    global db
+        SQLS[key] = val.replace('__TABLE_NAME__', table_name)
+    # check cache
     if filename in cache_db: # already open?
         db = cache_db[filename] # use_cache
-    else:
-        db = sqlite3.connect(filename)
-        cache_db[filename] = db
+        cur_filename = filename
+        return db
+    # connect to sqlite3
+    db = sqlite3.connect(filename, check_same_thread=False)
+    cache_db[filename] = db
     cur_filename = filename
     try:
         # create table
-        cur = db.cursor()
-        cur.execute(sqls['create'], [])
-        cur.execute(sqls['create_doc'], [])
+        db.executescript(SQLS['create'] + ';' + SQLS['create_doc'])
         # make cache keys
         get_keys(True)
+        return db
     except Exception as err:
         raise Exception('could not initalize database file: ' + str(err)) from err
-    finally:
-        cur.close()
-    return db
 
 def change_db(filename = ':memory:', table_name = 'kudb'):
     """Change Database"""
@@ -108,8 +109,8 @@ def close():
     """close database"""
     global db, cur_filename
     if db is not None:
-        del cache_db[cur_filename]
         db.close()
+        del cache_db[cur_filename]
     db = None
     cur_filename = ''
 
@@ -138,9 +139,9 @@ def get_key(key, default = '', file=None):
         raise Exception('please connect before using `get` method.')
     if key not in CACHE_KEYS:
         return default
+    cur = db.cursor()
     try:
-        cur = db.cursor()
-        cur.execute(sqls['select'], [key])
+        cur.execute(SQLS['select'], [key])
         values = cur.fetchone()
         return json.loads(values[0])
     except Exception as err:
@@ -156,7 +157,7 @@ def get_info(key, default = ''):
         return default
     try:
         cur = db.cursor()
-        cur.execute(sqls['select_info'], [key])
+        cur.execute(SQLS['select_info'], [key])
         values = cur.fetchone()
         return values
     except Exception as err:
@@ -181,10 +182,10 @@ def set_key(key, value, file=None):
     try:
         cur = db.cursor()
         if key in CACHE_KEYS:
-            cur.execute(sqls['update'], [value, int(time.time()), key])
+            cur.execute(SQLS['update'], [value, int(time.time()), key])
         else:
             cur.execute(
-                sqls['insert'],
+                SQLS['insert'],
                 [
                     key,
                     json.dumps(value, ensure_ascii=False),
@@ -193,10 +194,10 @@ def set_key(key, value, file=None):
                 ]
             )
             CACHE_KEYS[key] = True
-    except Exception as err:
-        raise Exception('could not read database: ' + str(err)) from err
-    finally:
         cur.close()
+        db.commit()
+    except Exception as err:
+        raise Exception('database could not write key: ' + str(err)) from err
 
 def delete_key(key):
     """delete key"""
@@ -205,12 +206,12 @@ def delete_key(key):
     try:
         cur = db.cursor()
         if key in CACHE_KEYS:
-            cur.execute(sqls['delete'], [key])
+            cur.execute(SQLS['delete'], [key])
             del CACHE_KEYS[key]
-    except Exception as err:
-        raise Exception('could not read database: ' + str(err)) from err
-    finally:
         cur.close()
+        db.commit()
+    except Exception as err:
+        raise Exception('database could not delete key: ' + str(err)) from err
 
 def get_keys(clear_cache = True):
     """
@@ -228,7 +229,7 @@ def get_keys(clear_cache = True):
     # Use Cache?
     if clear_cache or len(CACHE_KEYS) == 0:
         cur = db.cursor()
-        cur.execute(sqls['keys'])
+        cur.execute(SQLS['keys'])
         CACHE_KEYS = {}
         for i in cur.fetchall():
             CACHE_KEYS[i[0]] = True
@@ -248,30 +249,69 @@ def clear_keys():
         raise Exception('please connect before using `clear_keys` method.')
     try:
         cur = db.cursor()
-        cur.execute(sqls['clear'])
+        cur.execute(SQLS['clear'])
         CACHE_KEYS = {}
+        cur.close()
+        db.commit()
     except Exception as err:
         raise Exception('could not read database: ' + str(err)) from err
-    finally:
-        cur.close()
 
-def get_all():
+def count_doc(file=None):
     """
-    get all document's data
-    >>> _ = connect()
-    >>> clear()
-    >>> insert({'name': 'A'})
-    1
-    >>> insert({'name': 'B'})
-    2
-    >>> [a['name'] for a in get_all()]
-    ['A', 'B']
+    count doc
+
+    >>> clear(file=MEMORY_FILE)
+    >>> insert_many([{"name": "A"},{"name": "B"},{"name": "C"}])
+    >>> count_doc()
+    3
     """
+    if file is not None:
+        connect(file)
     if db is None:
-        raise Exception('please connect before using `all` method.')
+        raise Exception('please connect before using `count_doc` method.')
+    try:
+        cur = db.cursor()
+        cur.execute(SQLS['count_doc'])
+        val = cur.fetchone()
+        return val[0]
+    except Exception as err:
+        raise Exception('could not count docs:' + str(err)) from err
+
+def get_all(limit=None, order_asc=True, from_id=None, file=None):
+    """
+    get all doc
+    >>> clear(file=MEMORY_FILE)
+    >>> insert_many([{'name':'A'},{'name':'B'},{'name':'C'},{'name':'D'}])
+    >>> [a['name'] for a in get_all()]
+    ['A', 'B', 'C', 'D']
+    >>> [a['name'] for a in get_all(limit=2)]
+    ['A', 'B']
+    >>> [a['name'] for a in get_all(limit=2,from_id=3)]
+    ['C', 'D']
+    >>> [a['name'] for a in get_all(order_asc=False)]
+    ['D', 'C', 'B', 'A']
+    >>> [a['name'] for a in get_all(limit=2,order_asc=False)]
+    ['D', 'C']
+    """
+    # check parameters
+    if file is not None:
+        connect(file)
+    if db is None:
+        raise Exception('please connect before using `get_all` method.')
+    if limit is None:
+        limit = count_doc()
+    sql = SQLS['select_doc_asc']
+    if order_asc:
+        if from_id is None:
+            from_id = 1
+    else:
+        sql =  SQLS['select_doc_desc']
+        if from_id is None:
+            from_id = SQLITE_MAX_INT
+    # select doc
     result = []
     cur = db.cursor()
-    for row in cur.execute(sqls['select_doc']):
+    for row in cur.execute(sql, [from_id, limit]):
         values = json.loads(row[0])
         if isinstance(values, dict):
             values['id'] = row[1]
@@ -279,23 +319,34 @@ def get_all():
     cur.close()
     return result
 
-def recent(limit):
+def recent(limit=100, offset=0, order_asc=True):
     """
     get recent docs
     >>> clear(file=MEMORY_FILE)
     >>> insert_many( [{'name': 'A'}, {'name': 'B'}, {'name': 'C'}] )
     >>> [a['name'] for a in recent(2)]
-    ['C', 'B']
+    ['B', 'C']
+    >>> clear(file=MEMORY_FILE)
+    >>> insert_many( [1,2,3,4,5] )
+    >>> [v for v in recent(3)]
+    [3, 4, 5]
+    >>> [v for v in recent(limit=3, offset=3)]
+    [1, 2]
+    >>> [v for v in recent(limit=3, order_asc=False)]
+    [5, 4, 3]
     """
     if db is None:
         raise Exception('please connect before using `recent` method.')
     cur = db.cursor()
     result = []
-    for row in cur.execute(sqls['recent_doc'], [limit]):
+    for row in cur.execute(SQLS['recent_doc'], [limit, offset]):
         values = json.loads(row[0])
-        values['id'] = row[1]
+        if isinstance(values, dict):
+            values['id'] = row[1]
         result.append(values)
     cur.close()
+    if order_asc:
+        result.reverse()
     return result
 
 def get_by_id(id, def_value=None, file=None):
@@ -313,7 +364,7 @@ def get_by_id(id, def_value=None, file=None):
     if db is None:
         raise Exception('please connect before using `get` method.')
     cur = db.cursor()
-    cur.execute(sqls['get_doc_by_id'], [id])
+    cur.execute(SQLS['get_doc_by_id'], [id])
     data_one = cur.fetchone()
     if data_one is None:
         return def_value
@@ -357,10 +408,16 @@ def insert(values, file=None):
         connect(file)
     if db is None:
         raise Exception('please connect before using `insert` method.')
-    cur = db.cursor()
-    cur.execute(sqls['insert_doc'], [json.dumps(values), int(time.time()), int(time.time())])
-    cur.close()
-    return cur.lastrowid
+    try:
+        lastid = None
+        cur = db.cursor()
+        cur.execute(SQLS['insert_doc'], [json.dumps(values), int(time.time()), int(time.time())])
+        lastid = cur.lastrowid
+        cur.close()
+        db.commit()
+        return lastid
+    except Exception as err:
+        raise Exception('database insert error:' + str(err)) from err
 
 def insert_many(value_list, file=None):
     """
@@ -375,9 +432,22 @@ def insert_many(value_list, file=None):
     if file is not None:
         connect(file)
     if db is None:
-        raise Exception('please connect before using `insert` method.')
-    for row in value_list:
-        insert(row)
+        raise Exception('please connect before using `insert_many` method.')
+    if not isinstance(value_list, list):
+        raise Exception('please set the list type arguments to `insert_many` method.')
+    # make many values
+    t = int(time.time())
+    rows = []
+    for val in value_list:
+        rows.append([json.dumps(val), t, t])
+    # insert
+    try:
+        cur = db.cursor()
+        cur.executemany(SQLS['insert_doc'], rows)
+        cur.close()
+        db.commit()
+    except Exception as err:
+        raise Exception('database insert error:' + str(err)) from err
 
 def update(doc_id, values):
     """
@@ -392,9 +462,13 @@ def update(doc_id, values):
     """
     if db is None:
         raise Exception('please connect before using `update_doc` method.')
-    cur = db.cursor()
-    cur.execute(sqls['update_doc'], [json.dumps(values), int(time.time()), doc_id])
-    cur.close()
+    try:
+        cur = db.cursor()
+        cur.execute(SQLS['update_doc'], [json.dumps(values), int(time.time()), doc_id])
+        cur.close()
+        db.commit()
+    except Exception as err:
+        raise Exception('database update error:' + str(err)) from err
 
 def delete(id=None, key=None, file=None):
     """
@@ -417,8 +491,9 @@ def delete(id=None, key=None, file=None):
         raise Exception('please connect before using `delete` method.')
     if id is not None:
         cur = db.cursor()
-        cur.execute(sqls['delete_doc'], [id])
+        cur.execute(SQLS['delete_doc'], [id])
         cur.close()
+        db.commit()
         return
     if key is not None:
         delete_key(key)
@@ -426,23 +501,44 @@ def delete(id=None, key=None, file=None):
     raise Exception('should set id or key in `delete` method')
 
 def clear_doc(file=None):
-    """clear all"""
+    """
+    clear all doc
+
+    >>> clear(file=MEMORY_FILE)
+    >>> insert_many([1,2,3,4,5])
+    >>> count_doc()
+    5
+    >>> clear_doc()
+    >>> count_doc()
+    0
+    """
     if file is not None:
         connect(file)
     if db is None:
         raise Exception('please connect before using `clear_doc` method.')
     cur = db.cursor()
-    cur.execute(sqls['clear_doc'], [])
+    cur.execute(SQLS['clear_doc'], [])
     cur.close()
+    db.commit()
 
 def clear(file=None):
-    """clear all"""
+    """
+    clear doc and key-value-store
+
+    >>> clear(file=MEMORY_FILE)
+    >>> insert_many([1,2,3,4,5])
+    >>> count_doc()
+    5
+    >>> clear_doc()
+    >>> count_doc()
+    0
+    """
     if file is not None:
         connect(file)
     clear_keys()
     clear_doc()
 
-def find(callback):
+def find(callback, limit=None):
     """
     find doc by lambda
     >>> clear(file=MEMORY_FILE)
@@ -451,17 +547,23 @@ def find(callback):
     ['Bob']
     >>> [a['name'] for a in find(lambda v: v['age'] < 22)]
     ['Bob', 'Coo']
+    >>> clear()
+    >>> insert_many([1,2,3,4,5])
+    >>> [v for v in find(lambda c: c>=2, limit=3)]
+    [2, 3, 4]
     """
     if db is None:
         raise Exception('please connect before using `find` method.')
     result = []
     cur = db.cursor()
-    for row in cur.execute(sqls['select_doc']):
+    for row in cur.execute(SQLS['select_doc']):
         values = json.loads(row[0])
         if isinstance(values, dict):
             values['id'] = row[1]
         if callback(values):
             result.append(values)
+            if (limit is not None) and (len(result) >= limit):
+                break
     cur.close()
     return result
 
